@@ -2,11 +2,16 @@
 # Copyright (c) 2023 Airbyte, Inc., all rights reserved.
 #
 
+import sys
 from dataclasses import InitVar, dataclass
 from typing import Annotated, Any, Dict, List, Mapping, Optional, Union
 
 from airbyte_protocol_dataclasses.models import *  # noqa: F403  # Allow '*'
-from serpyco_rs.metadata import Alias
+
+if sys.platform == 'emscripten':
+    from serpyco.metadata import Alias
+else:
+    from serpyco_rs.metadata import Alias
 
 # ruff: noqa: F405  # ignore fuzzy import issues with 'import *'
 
