@@ -2,4 +2,8 @@ from .abstract_concurrent_file_based_cursor import AbstractConcurrentFileBasedCu
 from .file_based_concurrent_cursor import FileBasedConcurrentCursor
 from .file_based_final_state_cursor import FileBasedFinalStateCursor
 
-__all__ = ["AbstractConcurrentFileBasedCursor", "FileBasedConcurrentCursor", "FileBasedFinalStateCursor"]
+__all__ = [
+    "AbstractConcurrentFileBasedCursor",
+    "FileBasedConcurrentCursor",
+    "FileBasedFinalStateCursor",
+]
