@@ -2,10 +2,10 @@
 # Copyright (c) 2023 Airbyte, Inc., all rights reserved.
 #
 
+from datetime import timedelta
 from typing import Any, List, Mapping, Optional, Sequence, Tuple, Union
 
 import dpath
-import pendulum
 
 from airbyte_cdk.config_observation import (
     create_connector_config_control_message,
@@ -15,6 +15,11 @@ from airbyte_cdk.sources.message import MessageRepository, NoopMessageRepository
 from airbyte_cdk.sources.streams.http.requests_native_auth.abstract_oauth import (
     AbstractOauth2Authenticator,
 )
+from airbyte_cdk.utils.datetime_helpers import (
+    AirbyteDateTime,
+    ab_datetime_now,
+    ab_datetime_parse,
+)
 
 
 class Oauth2Authenticator(AbstractOauth2Authenticator):
@@ -34,7 +39,7 @@ class Oauth2Authenticator(AbstractOauth2Authenticator):
         client_secret_name: str = "client_secret",
         refresh_token_name: str = "refresh_token",
         scopes: List[str] | None = None,
-        token_expiry_date: pendulum.DateTime | None = None,
+        token_expiry_date: AirbyteDateTime | None = None,
         token_expiry_date_format: str | None = None,
         access_token_name: str = "access_token",
         expires_in_name: str = "expires_in",
@@ -46,7 +51,7 @@ class Oauth2Authenticator(AbstractOauth2Authenticator):
         refresh_token_error_status_codes: Tuple[int, ...] = (),
         refresh_token_error_key: str = "",
         refresh_token_error_values: Tuple[str, ...] = (),
-    ):
+    ) -> None:
         self._token_refresh_endpoint = token_refresh_endpoint
         self._client_secret_name = client_secret_name
         self._client_secret = client_secret
@@ -62,7 +67,7 @@ class Oauth2Authenticator(AbstractOauth2Authenticator):
         self._grant_type_name = grant_type_name
         self._grant_type = grant_type
 
-        self._token_expiry_date = token_expiry_date or pendulum.now().subtract(days=1)  # type: ignore [no-untyped-call]
+        self._token_expiry_date = token_expiry_date or (ab_datetime_now() - timedelta(days=1))
         self._token_expiry_date_format = token_expiry_date_format
         self._token_expiry_is_time_of_expiration = token_expiry_is_time_of_expiration
         self._access_token = None
@@ -112,11 +117,11 @@ class Oauth2Authenticator(AbstractOauth2Authenticator):
     def get_grant_type(self) -> str:
         return self._grant_type
 
-    def get_token_expiry_date(self) -> pendulum.DateTime:
+    def get_token_expiry_date(self) -> AirbyteDateTime:
         return self._token_expiry_date
 
-    def set_token_expiry_date(self, value: Union[str, int]) -> None:
-        self._token_expiry_date = self._parse_token_expiration_date(value)
+    def set_token_expiry_date(self, value: AirbyteDateTime) -> None:
+        self._token_expiry_date = value
 
     @property
     def token_expiry_is_time_of_expiration(self) -> bool:
@@ -170,7 +175,7 @@ class SingleUseRefreshTokenOauth2Authenticator(Oauth2Authenticator):
         refresh_token_error_status_codes: Tuple[int, ...] = (),
         refresh_token_error_key: str = "",
         refresh_token_error_values: Tuple[str, ...] = (),
-    ):
+    ) -> None:
         """
         Args:
             connector_config (Mapping[str, Any]): The full connector configuration
@@ -191,18 +196,12 @@ class SingleUseRefreshTokenOauth2Authenticator(Oauth2Authenticator):
             token_expiry_is_time_of_expiration bool: set True it if expires_in is returned as time of expiration instead of the number seconds until expiration
             message_repository (MessageRepository): the message repository used to emit logs on HTTP requests and control message on config update
         """
-        self._client_id = (
-            client_id  # type: ignore[assignment]  # Incorrect type for assignment
-            if client_id is not None
-            else dpath.get(connector_config, ("credentials", "client_id"))  # type: ignore[arg-type]
+        self._connector_config = connector_config
+        self._client_id: str = self._get_config_value_by_path(
+            ("credentials", "client_id"), client_id
         )
-        self._client_secret = (
-            client_secret  # type: ignore[assignment]  # Incorrect type for assignment
-            if client_secret is not None
-            else dpath.get(
-                connector_config,  # type: ignore[arg-type]
-                ("credentials", "client_secret"),
-            )
+        self._client_secret: str = self._get_config_value_by_path(
+            ("credentials", "client_secret"), client_secret
         )
         self._client_id_name = client_id_name
         self._client_secret_name = client_secret_name
@@ -217,9 +216,9 @@ class SingleUseRefreshTokenOauth2Authenticator(Oauth2Authenticator):
         super().__init__(
             token_refresh_endpoint=token_refresh_endpoint,
             client_id_name=self._client_id_name,
-            client_id=self.get_client_id(),
+            client_id=self._client_id,
             client_secret_name=self._client_secret_name,
-            client_secret=self.get_client_secret(),
+            client_secret=self._client_secret,
             refresh_token=self.get_refresh_token(),
             refresh_token_name=self._refresh_token_name,
             scopes=scopes,
@@ -237,76 +236,85 @@ class SingleUseRefreshTokenOauth2Authenticator(Oauth2Authenticator):
             refresh_token_error_values=refresh_token_error_values,
         )
 
-    def get_refresh_token_name(self) -> str:
-        return self._refresh_token_name
-
-    def get_client_id(self) -> str:
-        return self._client_id
-
-    def get_client_secret(self) -> str:
-        return self._client_secret
-
     @property
     def access_token(self) -> str:
-        return dpath.get(  # type: ignore[return-value]
-            self._connector_config,  # type: ignore[arg-type]
-            self._access_token_config_path,
-            default="",
-        )
+        """
+        Retrieve the access token from the configuration.
+
+        Returns:
+            str: The access token.
+        """
+        return self._get_config_value_by_path(self._access_token_config_path)  # type: ignore[return-value]
 
     @access_token.setter
     def access_token(self, new_access_token: str) -> None:
-        dpath.new(
-            self._connector_config,  # type: ignore[arg-type]
-            self._access_token_config_path,
-            new_access_token,
-        )
+        """
+        Sets a new access token.
+
+        Args:
+            new_access_token (str): The new access token to be set.
+        """
+        self._set_config_value_by_path(self._access_token_config_path, new_access_token)
 
     def get_refresh_token(self) -> str:
-        return dpath.get(  # type: ignore[return-value]
-            self._connector_config,  # type: ignore[arg-type]
-            self._refresh_token_config_path,
-            default="",
-        )
+        """
+        Retrieve the refresh token from the configuration.
+
+        This method fetches the refresh token using the configuration path specified
+        by `_refresh_token_config_path`.
+
+        Returns:
+            str: The refresh token as a string.
+        """
+        return self._get_config_value_by_path(self._refresh_token_config_path)  # type: ignore[return-value]
 
     def set_refresh_token(self, new_refresh_token: str) -> None:
-        dpath.new(
-            self._connector_config,  # type: ignore[arg-type]
-            self._refresh_token_config_path,
-            new_refresh_token,
-        )
+        """
+        Updates the refresh token in the configuration.
 
-    def get_token_expiry_date(self) -> pendulum.DateTime:
-        expiry_date = dpath.get(
-            self._connector_config,  # type: ignore[arg-type]
-            self._token_expiry_date_config_path,
-            default="",
-        )
-        return pendulum.now().subtract(days=1) if expiry_date == "" else pendulum.parse(expiry_date)  # type: ignore[arg-type, return-value, no-untyped-call]
+        Args:
+            new_refresh_token (str): The new refresh token to be set.
+        """
+        self._set_config_value_by_path(self._refresh_token_config_path, new_refresh_token)
 
-    def set_token_expiry_date(  # type: ignore[override]
-        self,
-        new_token_expiry_date: pendulum.DateTime,
-    ) -> None:
-        dpath.new(
-            self._connector_config,  # type: ignore[arg-type]
-            self._token_expiry_date_config_path,
-            str(new_token_expiry_date),
+    def get_token_expiry_date(self) -> AirbyteDateTime:
+        """
+        Retrieves the token expiry date from the configuration.
+
+        This method fetches the token expiry date from the configuration using the specified path.
+        If the expiry date is an empty string, it returns the current date and time minus one day.
+        Otherwise, it parses the expiry date string into an AirbyteDateTime object.
+
+        Returns:
+            AirbyteDateTime: The parsed or calculated token expiry date.
+
+        Raises:
+            TypeError: If the result is not an instance of AirbyteDateTime.
+        """
+        expiry_date = self._get_config_value_by_path(self._token_expiry_date_config_path)
+        result = (
+            ab_datetime_now() - timedelta(days=1)
+            if expiry_date == ""
+            else ab_datetime_parse(str(expiry_date))
+        )
+        if isinstance(result, AirbyteDateTime):
+            return result
+        raise TypeError("Invalid datetime conversion")
+
+    def set_token_expiry_date(self, new_token_expiry_date: AirbyteDateTime) -> None:  # type: ignore[override]
+        """
+        Sets the token expiry date in the configuration.
+
+        Args:
+            new_token_expiry_date (AirbyteDateTime): The new expiry date for the token.
+        """
+        self._set_config_value_by_path(
+            self._token_expiry_date_config_path, str(new_token_expiry_date)
         )
 
     def token_has_expired(self) -> bool:
         """Returns True if the token is expired"""
-        return pendulum.now("UTC") > self.get_token_expiry_date()
-
-    @staticmethod
-    def get_new_token_expiry_date(
-        access_token_expires_in: str,
-        token_expiry_date_format: str | None = None,
-    ) -> pendulum.DateTime:
-        if token_expiry_date_format:
-            return pendulum.from_format(access_token_expires_in, token_expiry_date_format)
-        else:
-            return pendulum.now("UTC").add(seconds=int(access_token_expires_in))
+        return ab_datetime_now() > self.get_token_expiry_date()
 
     def get_access_token(self) -> str:
         """Retrieve new access and refresh token if the access token has expired.
@@ -318,33 +326,85 @@ class SingleUseRefreshTokenOauth2Authenticator(Oauth2Authenticator):
             new_access_token, access_token_expires_in, new_refresh_token = (
                 self.refresh_access_token()
             )
-            new_token_expiry_date: pendulum.DateTime = self.get_new_token_expiry_date(
-                access_token_expires_in, self._token_expiry_date_format
-            )
             self.access_token = new_access_token
             self.set_refresh_token(new_refresh_token)
-            self.set_token_expiry_date(new_token_expiry_date)
-            # FIXME emit_configuration_as_airbyte_control_message as been deprecated in favor of package airbyte_cdk.sources.message
-            #  Usually, a class shouldn't care about the implementation details but to keep backward compatibility where we print the
-            #  message directly in the console, this is needed
-            if not isinstance(self._message_repository, NoopMessageRepository):
-                self._message_repository.emit_message(
-                    create_connector_config_control_message(self._connector_config)  # type: ignore[arg-type]
-                )
-            else:
-                emit_configuration_as_airbyte_control_message(self._connector_config)  # type: ignore[arg-type]
+            self.set_token_expiry_date(access_token_expires_in)
+            self._emit_control_message()
         return self.access_token
 
-    def refresh_access_token(  # type: ignore[override]  # Signature doesn't match base class
-        self,
-    ) -> Tuple[str, str, str]:
-        response_json = self._get_refresh_access_token_response()
+    def refresh_access_token(self) -> Tuple[str, AirbyteDateTime, str]:  # type: ignore[override]
+        """
+        Refreshes the access token by making a handled request and extracting the necessary token information.
+
+        Returns:
+            Tuple[str, str, str]: A tuple containing the new access token, token expiry date, and refresh token.
+        """
+        response_json = self._make_handled_request()
         return (
-            response_json[self.get_access_token_name()],
-            response_json[self.get_expires_in_name()],
-            response_json[self.get_refresh_token_name()],
+            self._extract_access_token(response_json),
+            self._extract_token_expiry_date(response_json),
+            self._extract_refresh_token(response_json),
         )
 
+    def _set_config_value_by_path(self, config_path: Union[str, Sequence[str]], value: Any) -> None:
+        """
+        Set a value in the connector configuration at the specified path.
+
+        Args:
+            config_path (Union[str, Sequence[str]]): The path within the configuration where the value should be set.
+                This can be a string representing a single key or a sequence of strings representing a nested path.
+            value (Any): The value to set at the specified path in the configuration.
+
+        Returns:
+            None
+        """
+        dpath.new(self._connector_config, config_path, value)  # type: ignore[arg-type]
+
+    def _get_config_value_by_path(
+        self, config_path: Union[str, Sequence[str]], default: Optional[str] = None
+    ) -> str | Any:
+        """
+        Retrieve a value from the connector configuration using a specified path.
+
+        Args:
+            config_path (Union[str, Sequence[str]]): The path to the desired configuration value. This can be a string or a sequence of strings.
+            default (Optional[str], optional): The default value to return if the specified path does not exist in the configuration. Defaults to None.
+
+        Returns:
+            Any: The value from the configuration at the specified path, or the default value if the path does not exist.
+        """
+        return dpath.get(
+            self._connector_config,  # type: ignore[arg-type]
+            config_path,
+            default=default if default is not None else "",
+        )
+
+    def _emit_control_message(self) -> None:
+        """
+        Emits a control message based on the connector configuration.
+
+        This method checks if the message repository is not a NoopMessageRepository.
+        If it is not, it emits a message using the message repository. Otherwise,
+        it falls back to emitting the configuration as an Airbyte control message
+        directly to the console for backward compatibility.
+
+        Note:
+            The function `emit_configuration_as_airbyte_control_message` has been deprecated
+            in favor of the package `airbyte_cdk.sources.message`.
+
+        Raises:
+            TypeError: If the argument types are incorrect.
+        """
+        # FIXME emit_configuration_as_airbyte_control_message as been deprecated in favor of package airbyte_cdk.sources.message
+        # Usually, a class shouldn't care about the implementation details but to keep backward compatibility where we print the
+        # message directly in the console, this is needed
+        if not isinstance(self._message_repository, NoopMessageRepository):
+            self._message_repository.emit_message(
+                create_connector_config_control_message(self._connector_config)  # type: ignore[arg-type]
+            )
+        else:
+            emit_configuration_as_airbyte_control_message(self._connector_config)  # type: ignore[arg-type]
+
     @property
     def _message_repository(self) -> MessageRepository:
         """
