@@ -17,6 +17,7 @@ than that, there are integrations point that are annoying to integrate with usin
 import json
 import logging
 import re
+import sys
 import tempfile
 import traceback
 from collections import deque
@@ -28,7 +29,6 @@ from typing import Any, List, Literal, Optional, Union, final, overload
 
 import orjson
 from pydantic import ValidationError as V2ValidationError
-from serpyco_rs import SchemaValidationError
 
 from airbyte_cdk.entrypoint import AirbyteEntrypoint
 from airbyte_cdk.exception_handler import assemble_uncaught_exception
@@ -50,8 +50,14 @@ from airbyte_cdk.models import (
 from airbyte_cdk.sources import Source
 from airbyte_cdk.test.models.scenario import ExpectedOutcome
 
+JsonValidationErrors: tuple[type[Exception], ...] = (orjson.JSONDecodeError,)
+# Conditionally import and create a union type for exception handling
+if sys.platform != "emscripten":
+    from serpyco_rs import SchemaValidationError
+
+    JsonValidationErrors = (orjson.JSONDecodeError, SchemaValidationError)
+
 
-@dataclass
 class AirbyteEntrypointException(Exception):
     """Exception raised for errors in the AirbyteEntrypoint execution.
 
@@ -65,6 +71,11 @@ class AirbyteEntrypointException(Exception):
             raise output.as_exception()
     """
 
+    message: str = ""
+
+    def __post_init__(self) -> None:
+        super().__init__(self.message)
+
 
 class EntrypointOutput:
     """A class to encapsulate the output of an Airbyte connector's execution.
@@ -114,7 +125,7 @@ class EntrypointOutput:
     def _parse_message(message: str) -> AirbyteMessage:
         try:
             return AirbyteMessageSerializer.load(orjson.loads(message))
-        except (orjson.JSONDecodeError, SchemaValidationError):
+        except JsonValidationErrors:
             # The platform assumes that logs that are not of AirbyteMessage format are log messages
             return AirbyteMessage(
                 type=Type.LOG, log=AirbyteLogMessage(level=Level.INFO, message=message)
