@@ -11,12 +11,12 @@ from airbyte_cdk.models import (
     AirbyteConnectionStatus,
     AirbyteErrorTraceMessage,
     AirbyteMessage,
-    AirbyteMessageSerializer,
     AirbyteTraceMessage,
     FailureType,
     Status,
     StreamDescriptor,
     TraceType,
+    ab_message_to_string,
 )
 from airbyte_cdk.models import Type as MessageType
 from airbyte_cdk.utils.airbyte_secrets_utils import filter_secrets
@@ -95,7 +95,7 @@ class AirbyteTracedException(Exception):
         Prints the exception as an AirbyteTraceMessage.
         Note that this will be called automatically on uncaught exceptions when using the airbyte_cdk entrypoint.
         """
-        message = orjson.dumps(AirbyteMessageSerializer.dump(self.as_airbyte_message())).decode()
+        message = ab_message_to_string(self.as_airbyte_message())
         filtered_message = filter_secrets(message)
         print(filtered_message)
 
